#include <thread>
#include <iostream>
#include <mutex>
#include <vector>

class atomic_counter {
public:
    void increment() {
        std::unique_lock<std::mutex> l(m_mutex);
        m_counter++;
    }
    
    void decrement() {
        std::unique_lock<std::mutex> l(m_mutex);
        m_counter--;
    }
    
    int value() {
        std::unique_lock<std::mutex> l(m_mutex);
        return m_counter;
    }
    
    bool increment_if_less_than(int val) {
        std::unique_lock<std::mutex> l(m_mutex);
        if (m_counter < val) {
            m_counter++;
            return true;
        }
        return false;
    }
    
private:
    int m_counter = 0;
    std::mutex m_mutex;
};

int main() {
    for (int i = 0; i < 1000000; ++i) {
        atomic_counter counter;

        auto thread_func = [&counter]() {
            while (counter.increment_if_less_than(5)) {
                // do something
            }
        };

        std::vector<std::thread> threads;
        for (int i = 0; i < 4; ++i) {
            threads.emplace_back(thread_func);
        }
        for (auto& thread : threads) {
            thread.join();
        }

        if (counter.value() != 5) {
            std::cout << counter.value() << '\n';
        }
    }
}
